# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.accounting_field import AccountingField
from ...types.additional_data_string import AdditionalDataString
from ...types.attachments import Attachments
from ...types.billitems import Billitems
from ...types.billstatus import Billstatus
from ...types.comments import Comments
from ...types.datenullable import Datenullable
from ...types.frequency import Frequency
from ...types.terms import Terms
from ...types.vendor_data import VendorData
from .bill_out_data_scheduled_options import BillOutDataScheduledOptions


class BillOutData(UniversalBaseModel):
    accounting_field_1: typing_extensions.Annotated[
        typing.Optional[AccountingField], FieldMetadata(alias="accountingField1")
    ] = None
    accounting_field_2: typing_extensions.Annotated[
        typing.Optional[AccountingField], FieldMetadata(alias="accountingField2")
    ] = None
    additional_data: typing_extensions.Annotated[
        typing.Optional[AdditionalDataString], FieldMetadata(alias="additionalData")
    ] = None
    attachments: typing.Optional[Attachments] = pydantic.Field(default=None)
    """
    An array of bill images. Attachments aren't required, but we strongly recommend including them. Including a bill image can make payouts smoother and prevent delays. You can include either the Base64-encoded file content, or you can include an fURL to a public file. The maximum file size for image uploads is 30 MB.
    """

    bill_date: typing_extensions.Annotated[typing.Optional[Datenullable], FieldMetadata(alias="billDate")] = (
        pydantic.Field(default=None)
    )
    """
    Date of bill. Accepted formats: YYYY-MM-DD, MM/DD/YYYY.
    """

    bill_items: typing_extensions.Annotated[typing.Optional[Billitems], FieldMetadata(alias="billItems")] = None
    bill_number: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="billNumber")] = pydantic.Field(
        default=None
    )
    """
    Unique identifier for the bill. Required when adding a bill.
    """

    comments: typing.Optional[Comments] = None
    discount: typing.Optional[float] = pydantic.Field(default=None)
    """
    Discount amount applied to the bill.
    """

    due_date: typing_extensions.Annotated[typing.Optional[Datenullable], FieldMetadata(alias="dueDate")] = (
        pydantic.Field(default=None)
    )
    """
    Due date of bill. Accepted formats: YYYY-MM-DD, MM/DD/YYYY.
    """

    end_date: typing_extensions.Annotated[typing.Optional[Datenullable], FieldMetadata(alias="endDate")] = (
        pydantic.Field(default=None)
    )
    """
    End Date for scheduled bills. Applied only in `Mode` = 1. Accepted formats: YYYY-MM-DD, MM/DD/YYYY
    """

    frequency: typing.Optional[Frequency] = pydantic.Field(default=None)
    """
    Frequency for scheduled bills. Applied only in `Mode` = 1.
    """

    lot_number: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="lotNumber")] = pydantic.Field(
        default=None
    )
    """
    Lot number associated with the bill.
    """

    mode: typing.Optional[int] = pydantic.Field(default=None)
    """
    Bill mode: value `0` for one-time bills, `1` for scheduled bills.
    """

    net_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="netAmount")] = pydantic.Field(
        default=None
    )
    """
    Net Amount owed in bill. Required when adding a bill.
    """

    scheduled_options: typing_extensions.Annotated[
        typing.Optional[BillOutDataScheduledOptions], FieldMetadata(alias="scheduledOptions")
    ] = pydantic.Field(default=None)
    """
    Options for scheduled bills.
    """

    status: typing.Optional[Billstatus] = None
    terms: typing.Optional[Terms] = None
    total_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="totalAmount")] = (
        pydantic.Field(default=None)
    )
    """
    Total amount of the bill.
    """

    vendor: typing.Optional[VendorData] = pydantic.Field(default=None)
    """
    The vendor associated with the bill. Although you can create a vendor in a create bill request, Payabli recommends creating a vendor separately and passing a valid `vendorNumber` here. At minimum, the `vendorNumber` is required. 
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .method_element_settings_apple_pay_button_style import MethodElementSettingsApplePayButtonStyle
from .method_element_settings_apple_pay_button_type import MethodElementSettingsApplePayButtonType
from .method_element_settings_apple_pay_language import MethodElementSettingsApplePayLanguage


class MethodElementSettingsApplePay(UniversalBaseModel):
    button_style: typing_extensions.Annotated[
        typing.Optional[MethodElementSettingsApplePayButtonStyle], FieldMetadata(alias="buttonStyle")
    ] = pydantic.Field(default=None)
    """
    The Apple Pay button style. See [Apple Pay Button Style](/developers/developer-guides/hosted-payment-page-apple-pay#param-applepay-button-style) for more information.
    """

    button_type: typing_extensions.Annotated[
        typing.Optional[MethodElementSettingsApplePayButtonType], FieldMetadata(alias="buttonType")
    ] = pydantic.Field(default=None)
    """
    The text on Apple Pay button. See [Apple Pay Button Type](/developers/developer-guides/hosted-payment-page-apple-pay#param-applepay-button-type) for more information.
    """

    language: typing.Optional[MethodElementSettingsApplePayLanguage] = pydantic.Field(default=None)
    """
    The Apple Pay button locale. See [Apple Pay Button Language](/developers/developer-guides/hosted-payment-page-apple-pay#param-applepay-language) for more information.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .ocr_attachment import OcrAttachment
from .ocr_bill_item import OcrBillItem
from .ocr_bill_item_additional_data import OcrBillItemAdditionalData
from .ocr_vendor import OcrVendor


class OcrResultData(UniversalBaseModel):
    bill_number: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="billNumber")] = None
    net_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="netAmount")] = None
    bill_date: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="billDate")] = None
    due_date: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="dueDate")] = None
    comments: typing.Optional[str] = None
    bill_items: typing_extensions.Annotated[
        typing.Optional[typing.List[OcrBillItem]], FieldMetadata(alias="billItems")
    ] = None
    mode: typing.Optional[int] = None
    accounting_field_1: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="accountingField1")] = (
        None
    )
    accounting_field_2: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="accountingField2")] = (
        None
    )
    additional_data: typing_extensions.Annotated[
        typing.Optional[OcrBillItemAdditionalData], FieldMetadata(alias="additionalData")
    ] = None
    vendor: typing.Optional[OcrVendor] = None
    end_date: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="endDate")] = None
    frequency: typing.Optional[str] = None
    terms: typing.Optional[str] = None
    status: typing.Optional[int] = None
    lot_number: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="lotNumber")] = None
    attachments: typing.Optional[typing.List[OcrAttachment]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class StatisticsVendorQueryRecord(UniversalBaseModel):
    stat_x: typing_extensions.Annotated[str, FieldMetadata(alias="statX")] = pydantic.Field()
    """
    Statistical grouping identifier
    """

    active: int = pydantic.Field()
    """
    Number of active transactions
    """

    active_volume: typing_extensions.Annotated[float, FieldMetadata(alias="activeVolume")] = pydantic.Field()
    """
    Volume of active transactions
    """

    sent_to_approval: typing_extensions.Annotated[int, FieldMetadata(alias="sentToApproval")] = pydantic.Field()
    """
    Number of transactions sent to approval
    """

    sent_to_approval_volume: typing_extensions.Annotated[float, FieldMetadata(alias="sentToApprovalVolume")] = (
        pydantic.Field()
    )
    """
    Volume of transactions sent to approval
    """

    to_approval: typing_extensions.Annotated[int, FieldMetadata(alias="toApproval")] = pydantic.Field()
    """
    Number of transactions to approval
    """

    to_approval_volume: typing_extensions.Annotated[float, FieldMetadata(alias="toApprovalVolume")] = pydantic.Field()
    """
    Volume of transactions to approval
    """

    approved: int = pydantic.Field()
    """
    Number of approved transactions
    """

    approved_volume: typing_extensions.Annotated[float, FieldMetadata(alias="approvedVolume")] = pydantic.Field()
    """
    Volume of approved transactions
    """

    disapproved: int = pydantic.Field()
    """
    Number of disapproved transactions
    """

    disapproved_volume: typing_extensions.Annotated[float, FieldMetadata(alias="disapprovedVolume")] = pydantic.Field()
    """
    Volume of disapproved transactions
    """

    cancelled: int = pydantic.Field()
    """
    Number of cancelled transactions
    """

    cancelled_volume: typing_extensions.Annotated[float, FieldMetadata(alias="cancelledVolume")] = pydantic.Field()
    """
    Volume of cancelled transactions
    """

    in_transit: typing_extensions.Annotated[int, FieldMetadata(alias="inTransit")] = pydantic.Field()
    """
    Number of transactions in transit
    """

    in_transit_volume: typing_extensions.Annotated[float, FieldMetadata(alias="inTransitVolume")] = pydantic.Field()
    """
    Volume of transactions in transit
    """

    paid: int = pydantic.Field()
    """
    Number of paid transactions
    """

    paid_volume: typing_extensions.Annotated[float, FieldMetadata(alias="paidVolume")] = pydantic.Field()
    """
    Volume of paid transactions
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .created_at import CreatedAt
from .dbaname import Dbaname
from .entrypointfield import Entrypointfield
from .item_commodity_code import ItemCommodityCode
from .item_description import ItemDescription
from .item_product_code import ItemProductCode
from .item_product_name import ItemProductName
from .item_unitof_measure import ItemUnitofMeasure
from .last_modified import LastModified
from .legalname import Legalname
from .org_parent_name import OrgParentName
from .page_identifier import PageIdentifier


class LineItemQueryRecord(UniversalBaseModel):
    created_at: typing_extensions.Annotated[typing.Optional[CreatedAt], FieldMetadata(alias="createdAt")] = (
        pydantic.Field(default=None)
    )
    """
    Timestamp of when line item was created, in UTC.
    """

    id: typing.Optional[int] = pydantic.Field(default=None)
    """
    Identifier of line item.
    """

    item_categories: typing_extensions.Annotated[
        typing.Optional[typing.List[typing.Optional[str]]], FieldMetadata(alias="itemCategories")
    ] = pydantic.Field(default=None)
    """
    Array of tags classifying item or product.
    """

    item_commodity_code: typing_extensions.Annotated[
        typing.Optional[ItemCommodityCode], FieldMetadata(alias="itemCommodityCode")
    ] = None
    item_cost: typing_extensions.Annotated[float, FieldMetadata(alias="itemCost")] = pydantic.Field()
    """
    Item or product price per unit.
    """

    item_description: typing_extensions.Annotated[
        typing.Optional[ItemDescription], FieldMetadata(alias="itemDescription")
    ] = None
    item_mode: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="itemMode")] = pydantic.Field(
        default=None
    )
    """
    Internal class of item or product: value '0' is only for invoices , '1' for bills, and '2' common for both.
    """

    item_product_code: typing_extensions.Annotated[
        typing.Optional[ItemProductCode], FieldMetadata(alias="itemProductCode")
    ] = None
    item_product_name: typing_extensions.Annotated[
        typing.Optional[ItemProductName], FieldMetadata(alias="itemProductName")
    ] = None
    item_qty: typing_extensions.Annotated[int, FieldMetadata(alias="itemQty")] = pydantic.Field()
    """
    Quantity of item or product.
    """

    item_unit_of_measure: typing_extensions.Annotated[
        typing.Optional[ItemUnitofMeasure], FieldMetadata(alias="itemUnitOfMeasure")
    ] = None
    last_updated: typing_extensions.Annotated[typing.Optional[LastModified], FieldMetadata(alias="lastUpdated")] = (
        pydantic.Field(default=None)
    )
    """
    Timestamp of when the line item was updated, in UTC.
    """

    pageidentifier: typing.Optional[PageIdentifier] = None
    parent_org_name: typing_extensions.Annotated[
        typing.Optional[OrgParentName], FieldMetadata(alias="ParentOrgName")
    ] = pydantic.Field(default=None)
    """
    The name of the paypoint's parent organization.
    """

    paypoint_dbaname: typing_extensions.Annotated[typing.Optional[Dbaname], FieldMetadata(alias="PaypointDbaname")] = (
        pydantic.Field(default=None)
    )
    """
    The paypoint's DBA name.
    """

    paypoint_entryname: typing_extensions.Annotated[
        typing.Optional[Entrypointfield], FieldMetadata(alias="PaypointEntryname")
    ] = pydantic.Field(default=None)
    """
    The paypoint's entryname (entrypoint) value.
    """

    paypoint_legalname: typing_extensions.Annotated[
        typing.Optional[Legalname], FieldMetadata(alias="PaypointLegalname")
    ] = pydantic.Field(default=None)
    """
    The paypoint's legal name.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .enabled import Enabled
from .order import Order


class ContactElement(UniversalBaseModel):
    email_label: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="emailLabel")] = pydantic.Field(
        default=None
    )
    """
    Custom content for email
    """

    enabled: typing.Optional[Enabled] = None
    header: typing.Optional[str] = pydantic.Field(default=None)
    """
    Header text for section
    """

    order: typing.Optional[Order] = None
    payment_icons: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="paymentIcons")] = (
        pydantic.Field(default=None)
    )
    """
    Flag indicating if icons for accepted card brands will be shown
    """

    phone_label: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="phoneLabel")] = pydantic.Field(
        default=None
    )
    """
    Custom content for phone number
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .accounting_field import AccountingField
from .additional_data_string import AdditionalDataString
from .attachments import Attachments
from .comments import Comments
from .datenullable import Datenullable
from .invoice_number import InvoiceNumber
from .net_amountstring import NetAmountstring
from .terms import Terms


class BillPayOutDataRequest(UniversalBaseModel):
    bill_id: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="billId")] = pydantic.Field(
        default=None
    )
    """
    Bill ID in Payabli.
    """

    comments: typing.Optional[Comments] = pydantic.Field(default=None)
    """
    Any comments about bill. **For managed payouts, this field has a limit of 100 characters**.
    """

    due_date: typing_extensions.Annotated[typing.Optional[Datenullable], FieldMetadata(alias="dueDate")] = (
        pydantic.Field(default=None)
    )
    """
    Bill due date in format YYYY-MM-DD or MM/DD/YYYY.
    """

    invoice_date: typing_extensions.Annotated[typing.Optional[Datenullable], FieldMetadata(alias="invoiceDate")] = (
        pydantic.Field(default=None)
    )
    """
    Bill date in format YYYY-MM-DD or MM/DD/YYYY.
    """

    invoice_number: typing_extensions.Annotated[
        typing.Optional[InvoiceNumber], FieldMetadata(alias="invoiceNumber")
    ] = pydantic.Field(default=None)
    """
    Custom number identifying the bill. Must be unique in paypoint. **Required** for new bill and when `billId` isn't provided.
    """

    net_amount: typing_extensions.Annotated[typing.Optional[NetAmountstring], FieldMetadata(alias="netAmount")] = (
        pydantic.Field(default=None)
    )
    """
    Net Amount owed in bill. Required when adding a bill.
    """

    discount: typing.Optional[str] = pydantic.Field(default=None)
    """
    Bill discount amount.
    """

    terms: typing.Optional[Terms] = pydantic.Field(default=None)
    """
    Description of payment terms.
    """

    accounting_field_1: typing_extensions.Annotated[
        typing.Optional[AccountingField], FieldMetadata(alias="accountingField1")
    ] = None
    accounting_field_2: typing_extensions.Annotated[
        typing.Optional[AccountingField], FieldMetadata(alias="accountingField2")
    ] = None
    additional_data: typing_extensions.Annotated[
        typing.Optional[AdditionalDataString], FieldMetadata(alias="additionalData")
    ] = None
    attachments: typing.Optional[Attachments] = pydantic.Field(default=None)
    """
    Bill image attachment. Send the bill image as Base64-encoded string, or as a publicly accessible link. For full details on using this field with a payout authorization, see [the documentation](/developers/developer-guides/pay-out-manage-payouts).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .additional_data import AdditionalData
from .page_content import PageContent
from .page_identifier import PageIdentifier
from .page_setting import PageSetting
from .payabli_credentials import PayabliCredentials
from .receipt_content import ReceiptContent
from .subdomain import Subdomain


class PayabliPages(UniversalBaseModel):
    additional_data: typing_extensions.Annotated[
        typing.Optional[AdditionalData], FieldMetadata(alias="AdditionalData")
    ] = None
    credentials: typing.Optional[typing.List[PayabliCredentials]] = pydantic.Field(default=None)
    """
    Array of credential objects with active services for the page
    """

    last_access: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="lastAccess")] = (
        pydantic.Field(default=None)
    )
    """
    Timestamp of last access to page structure
    """

    page_content: typing_extensions.Annotated[typing.Optional[PageContent], FieldMetadata(alias="pageContent")] = (
        pydantic.Field(default=None)
    )
    """
    Sections of page
    """

    page_identifier: typing_extensions.Annotated[
        typing.Optional[PageIdentifier], FieldMetadata(alias="pageIdentifier")
    ] = None
    page_settings: typing_extensions.Annotated[typing.Optional[PageSetting], FieldMetadata(alias="pageSettings")] = (
        pydantic.Field(default=None)
    )
    """
    Settings of page
    """

    published: typing.Optional[int] = pydantic.Field(default=None)
    """
    Flag indicating if page is active to accept payments. `0` for false, `1` for true.
    """

    receipt_content: typing_extensions.Annotated[
        typing.Optional[ReceiptContent], FieldMetadata(alias="receiptContent")
    ] = pydantic.Field(default=None)
    """
    Sections of payment receipt
    """

    subdomain: typing.Optional[Subdomain] = pydantic.Field(default=None)
    """
    Page identifier. Must be unique in platform.
    """

    total_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="totalAmount")] = (
        pydantic.Field(default=None)
    )
    """
    Total amount to pay in this page
    """

    validation_code: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="validationCode")] = (
        pydantic.Field(default=None)
    )
    """
    Base64 encoded image of CAPTCHA associated to this page load
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.ach_validation import AchValidation
from ..types.entrypointfield import Entrypointfield
from ..types.force_customer_creation import ForceCustomerCreation
from ..types.idempotency_key import IdempotencyKey
from ..types.payabli_api_response_paymethod_delete import PayabliApiResponsePaymethodDelete
from ..types.payor_data_request import PayorDataRequest
from ..types.source import Source
from ..types.subdomain import Subdomain
from .raw_client import AsyncRawTokenStorageClient, RawTokenStorageClient
from .types.add_method_response import AddMethodResponse
from .types.create_anonymous import CreateAnonymous
from .types.get_method_response import GetMethodResponse
from .types.request_token_storage_payment_method import RequestTokenStoragePaymentMethod
from .types.temporary import Temporary

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class TokenStorageClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawTokenStorageClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawTokenStorageClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawTokenStorageClient
        """
        return self._raw_client

    def add_method(
        self,
        *,
        create_anonymous: CreateAnonymous,
        temporary: Temporary,
        ach_validation: typing.Optional[AchValidation] = None,
        force_customer_creation: typing.Optional[ForceCustomerCreation] = None,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        customer_data: typing.Optional[PayorDataRequest] = OMIT,
        entry_point: typing.Optional[Entrypointfield] = OMIT,
        fallback_auth: typing.Optional[bool] = OMIT,
        fallback_auth_amount: typing.Optional[int] = OMIT,
        method_description: typing.Optional[str] = OMIT,
        payment_method: typing.Optional[RequestTokenStoragePaymentMethod] = OMIT,
        source: typing.Optional[Source] = OMIT,
        subdomain: typing.Optional[Subdomain] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddMethodResponse:
        """
        Saves a payment method for reuse. This call exchanges sensitive payment information for a token that can be used to process future transactions. The `ReferenceId` value in the response is the `storedMethodId` to use with transactions.

        Parameters
        ----------
        create_anonymous : CreateAnonymous

        temporary : Temporary

        ach_validation : typing.Optional[AchValidation]

        force_customer_creation : typing.Optional[ForceCustomerCreation]

        idempotency_key : typing.Optional[IdempotencyKey]

        customer_data : typing.Optional[PayorDataRequest]
            Object describing the Customer/Payor owner of payment method. Required for POST requests. Which fields are required depends on the paypoint's custom identifier settings.

        entry_point : typing.Optional[Entrypointfield]
            Entrypoint identifier. Required for POST requests.

        fallback_auth : typing.Optional[bool]
            When `true`, if tokenization fails, Payabli will attempt an authorization transaction to request a permanent token for the card. If the authorization is successful, the card will be tokenized and the authorization will be voided automatically.

        fallback_auth_amount : typing.Optional[int]
            The amount for the `fallbackAuth` transaction. Defaults to one dollar (`100`).

        method_description : typing.Optional[str]
            Custom description for stored payment method.

        payment_method : typing.Optional[RequestTokenStoragePaymentMethod]
            Information about the payment method for the transaction.

        source : typing.Optional[Source]
            Custom identifier to indicate the source for the request

        subdomain : typing.Optional[Subdomain]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddMethodResponse
            Success

        Examples
        --------
        from payabli import PayorDataRequest, payabli
        from payabli.token_storage import TokenizeCard

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.token_storage.add_method(
            customer_data=PayorDataRequest(
                customer_id=4440,
            ),
            entry_point="f743aed24a",
            fallback_auth=True,
            payment_method=TokenizeCard(
                cardcvv="123",
                cardexp="02/25",
                card_holder="John Doe",
                cardnumber="4111111111111111",
                cardzip="12345",
                method="card",
            ),
        )
        """
        _response = self._raw_client.add_method(
            create_anonymous=create_anonymous,
            temporary=temporary,
            ach_validation=ach_validation,
            force_customer_creation=force_customer_creation,
            idempotency_key=idempotency_key,
            customer_data=customer_data,
            entry_point=entry_point,
            fallback_auth=fallback_auth,
            fallback_auth_amount=fallback_auth_amount,
            method_description=method_description,
            payment_method=payment_method,
            source=source,
            subdomain=subdomain,
            request_options=request_options,
        )
        return _response.data

    def get_method(
        self,
        method_id: str,
        *,
        card_expiration_format: typing.Optional[int] = None,
        include_temporary: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetMethodResponse:
        """
        Retrieves details for a saved payment method.

        Parameters
        ----------
        method_id : str
            The saved payment method ID.

        card_expiration_format : typing.Optional[int]
            Format for card expiration dates in the response.

            Accepted values:

            - 0: default, no formatting. Expiration dates are returned in the format they're saved in.

            - 1: MMYY

            - 2: MM/YY

        include_temporary : typing.Optional[bool]
            When `true`, the request will include temporary tokens in the search and return details for a matching temporary token. The default behavior searches only for permanent tokens.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetMethodResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.token_storage.get_method(
            method_id="32-8877drt00045632-678",
            card_expiration_format=1,
            include_temporary=False,
        )
        """
        _response = self._raw_client.get_method(
            method_id,
            card_expiration_format=card_expiration_format,
            include_temporary=include_temporary,
            request_options=request_options,
        )
        return _response.data

    def remove_method(
        self, method_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliApiResponsePaymethodDelete:
        """
        Deletes a saved payment method.

        Parameters
        ----------
        method_id : str
            The saved payment method ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponsePaymethodDelete
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.token_storage.remove_method(
            method_id="32-8877drt00045632-678",
        )
        """
        _response = self._raw_client.remove_method(method_id, request_options=request_options)
        return _response.data

    def update_method(
        self,
        method_id: str,
        *,
        ach_validation: typing.Optional[AchValidation] = None,
        customer_data: typing.Optional[PayorDataRequest] = OMIT,
        entry_point: typing.Optional[Entrypointfield] = OMIT,
        fallback_auth: typing.Optional[bool] = OMIT,
        fallback_auth_amount: typing.Optional[int] = OMIT,
        method_description: typing.Optional[str] = OMIT,
        payment_method: typing.Optional[RequestTokenStoragePaymentMethod] = OMIT,
        source: typing.Optional[Source] = OMIT,
        subdomain: typing.Optional[Subdomain] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayabliApiResponsePaymethodDelete:
        """
        Updates a saved payment method.

        Parameters
        ----------
        method_id : str
            The saved payment method ID.

        ach_validation : typing.Optional[AchValidation]

        customer_data : typing.Optional[PayorDataRequest]
            Object describing the Customer/Payor owner of payment method. Required for POST requests. Which fields are required depends on the paypoint's custom identifier settings.

        entry_point : typing.Optional[Entrypointfield]
            Entrypoint identifier. Required for POST requests.

        fallback_auth : typing.Optional[bool]
            When `true`, if tokenization fails, Payabli will attempt an authorization transaction to request a permanent token for the card. If the authorization is successful, the card will be tokenized and the authorization will be voided automatically.

        fallback_auth_amount : typing.Optional[int]
            The amount for the `fallbackAuth` transaction. Defaults to one dollar (`100`).

        method_description : typing.Optional[str]
            Custom description for stored payment method.

        payment_method : typing.Optional[RequestTokenStoragePaymentMethod]
            Information about the payment method for the transaction.

        source : typing.Optional[Source]
            Custom identifier to indicate the source for the request

        subdomain : typing.Optional[Subdomain]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponsePaymethodDelete
            Success

        Examples
        --------
        from payabli import PayorDataRequest, payabli
        from payabli.token_storage import TokenizeCard

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.token_storage.update_method(
            method_id="32-8877drt00045632-678",
            customer_data=PayorDataRequest(
                customer_id=4440,
            ),
            entry_point="f743aed24a",
            fallback_auth=True,
            payment_method=TokenizeCard(
                cardcvv="123",
                cardexp="02/25",
                card_holder="John Doe",
                cardnumber="4111111111111111",
                cardzip="12345",
                method="card",
            ),
        )
        """
        _response = self._raw_client.update_method(
            method_id,
            ach_validation=ach_validation,
            customer_data=customer_data,
            entry_point=entry_point,
            fallback_auth=fallback_auth,
            fallback_auth_amount=fallback_auth_amount,
            method_description=method_description,
            payment_method=payment_method,
            source=source,
            subdomain=subdomain,
            request_options=request_options,
        )
        return _response.data


class AsyncTokenStorageClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawTokenStorageClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawTokenStorageClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawTokenStorageClient
        """
        return self._raw_client

    async def add_method(
        self,
        *,
        create_anonymous: CreateAnonymous,
        temporary: Temporary,
        ach_validation: typing.Optional[AchValidation] = None,
        force_customer_creation: typing.Optional[ForceCustomerCreation] = None,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        customer_data: typing.Optional[PayorDataRequest] = OMIT,
        entry_point: typing.Optional[Entrypointfield] = OMIT,
        fallback_auth: typing.Optional[bool] = OMIT,
        fallback_auth_amount: typing.Optional[int] = OMIT,
        method_description: typing.Optional[str] = OMIT,
        payment_method: typing.Optional[RequestTokenStoragePaymentMethod] = OMIT,
        source: typing.Optional[Source] = OMIT,
        subdomain: typing.Optional[Subdomain] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddMethodResponse:
        """
        Saves a payment method for reuse. This call exchanges sensitive payment information for a token that can be used to process future transactions. The `ReferenceId` value in the response is the `storedMethodId` to use with transactions.

        Parameters
        ----------
        create_anonymous : CreateAnonymous

        temporary : Temporary

        ach_validation : typing.Optional[AchValidation]

        force_customer_creation : typing.Optional[ForceCustomerCreation]

        idempotency_key : typing.Optional[IdempotencyKey]

        customer_data : typing.Optional[PayorDataRequest]
            Object describing the Customer/Payor owner of payment method. Required for POST requests. Which fields are required depends on the paypoint's custom identifier settings.

        entry_point : typing.Optional[Entrypointfield]
            Entrypoint identifier. Required for POST requests.

        fallback_auth : typing.Optional[bool]
            When `true`, if tokenization fails, Payabli will attempt an authorization transaction to request a permanent token for the card. If the authorization is successful, the card will be tokenized and the authorization will be voided automatically.

        fallback_auth_amount : typing.Optional[int]
            The amount for the `fallbackAuth` transaction. Defaults to one dollar (`100`).

        method_description : typing.Optional[str]
            Custom description for stored payment method.

        payment_method : typing.Optional[RequestTokenStoragePaymentMethod]
            Information about the payment method for the transaction.

        source : typing.Optional[Source]
            Custom identifier to indicate the source for the request

        subdomain : typing.Optional[Subdomain]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddMethodResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli, PayorDataRequest
        from payabli.token_storage import TokenizeCard

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.token_storage.add_method(
                customer_data=PayorDataRequest(
                    customer_id=4440,
                ),
                entry_point="f743aed24a",
                fallback_auth=True,
                payment_method=TokenizeCard(
                    cardcvv="123",
                    cardexp="02/25",
                    card_holder="John Doe",
                    cardnumber="4111111111111111",
                    cardzip="12345",
                    method="card",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.add_method(
            create_anonymous=create_anonymous,
            temporary=temporary,
            ach_validation=ach_validation,
            force_customer_creation=force_customer_creation,
            idempotency_key=idempotency_key,
            customer_data=customer_data,
            entry_point=entry_point,
            fallback_auth=fallback_auth,
            fallback_auth_amount=fallback_auth_amount,
            method_description=method_description,
            payment_method=payment_method,
            source=source,
            subdomain=subdomain,
            request_options=request_options,
        )
        return _response.data

    async def get_method(
        self,
        method_id: str,
        *,
        card_expiration_format: typing.Optional[int] = None,
        include_temporary: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetMethodResponse:
        """
        Retrieves details for a saved payment method.

        Parameters
        ----------
        method_id : str
            The saved payment method ID.

        card_expiration_format : typing.Optional[int]
            Format for card expiration dates in the response.

            Accepted values:

            - 0: default, no formatting. Expiration dates are returned in the format they're saved in.

            - 1: MMYY

            - 2: MM/YY

        include_temporary : typing.Optional[bool]
            When `true`, the request will include temporary tokens in the search and return details for a matching temporary token. The default behavior searches only for permanent tokens.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetMethodResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.token_storage.get_method(
                method_id="32-8877drt00045632-678",
                card_expiration_format=1,
                include_temporary=False,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_method(
            method_id,
            card_expiration_format=card_expiration_format,
            include_temporary=include_temporary,
            request_options=request_options,
        )
        return _response.data

    async def remove_method(
        self, method_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliApiResponsePaymethodDelete:
        """
        Deletes a saved payment method.

        Parameters
        ----------
        method_id : str
            The saved payment method ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponsePaymethodDelete
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.token_storage.remove_method(
                method_id="32-8877drt00045632-678",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.remove_method(method_id, request_options=request_options)
        return _response.data

    async def update_method(
        self,
        method_id: str,
        *,
        ach_validation: typing.Optional[AchValidation] = None,
        customer_data: typing.Optional[PayorDataRequest] = OMIT,
        entry_point: typing.Optional[Entrypointfield] = OMIT,
        fallback_auth: typing.Optional[bool] = OMIT,
        fallback_auth_amount: typing.Optional[int] = OMIT,
        method_description: typing.Optional[str] = OMIT,
        payment_method: typing.Optional[RequestTokenStoragePaymentMethod] = OMIT,
        source: typing.Optional[Source] = OMIT,
        subdomain: typing.Optional[Subdomain] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayabliApiResponsePaymethodDelete:
        """
        Updates a saved payment method.

        Parameters
        ----------
        method_id : str
            The saved payment method ID.

        ach_validation : typing.Optional[AchValidation]

        customer_data : typing.Optional[PayorDataRequest]
            Object describing the Customer/Payor owner of payment method. Required for POST requests. Which fields are required depends on the paypoint's custom identifier settings.

        entry_point : typing.Optional[Entrypointfield]
            Entrypoint identifier. Required for POST requests.

        fallback_auth : typing.Optional[bool]
            When `true`, if tokenization fails, Payabli will attempt an authorization transaction to request a permanent token for the card. If the authorization is successful, the card will be tokenized and the authorization will be voided automatically.

        fallback_auth_amount : typing.Optional[int]
            The amount for the `fallbackAuth` transaction. Defaults to one dollar (`100`).

        method_description : typing.Optional[str]
            Custom description for stored payment method.

        payment_method : typing.Optional[RequestTokenStoragePaymentMethod]
            Information about the payment method for the transaction.

        source : typing.Optional[Source]
            Custom identifier to indicate the source for the request

        subdomain : typing.Optional[Subdomain]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponsePaymethodDelete
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli, PayorDataRequest
        from payabli.token_storage import TokenizeCard

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.token_storage.update_method(
                method_id="32-8877drt00045632-678",
                customer_data=PayorDataRequest(
                    customer_id=4440,
                ),
                entry_point="f743aed24a",
                fallback_auth=True,
                payment_method=TokenizeCard(
                    cardcvv="123",
                    cardexp="02/25",
                    card_holder="John Doe",
                    cardnumber="4111111111111111",
                    cardzip="12345",
                    method="card",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_method(
            method_id,
            ach_validation=ach_validation,
            customer_data=customer_data,
            entry_point=entry_point,
            fallback_auth=fallback_auth,
            fallback_auth_amount=fallback_auth_amount,
            method_description=method_description,
            payment_method=payment_method,
            source=source,
            subdomain=subdomain,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.is_success import IsSuccess
from ...types.page_identifier import PageIdentifier
from ...types.response_text import ResponseText
from ...types.responsecode import Responsecode
from ...types.responsedatanonobject import Responsedatanonobject
from ...types.room_id_not_in_use import RoomIdNotInUse


class InvoiceResponseWithoutData(UniversalBaseModel):
    """
    Response schema for invoice operations.
    """

    is_success: typing_extensions.Annotated[IsSuccess, FieldMetadata(alias="isSuccess")]
    response_code: typing_extensions.Annotated[Responsecode, FieldMetadata(alias="responseCode")]
    response_data: typing_extensions.Annotated[Responsedatanonobject, FieldMetadata(alias="responseData")] = (
        pydantic.Field()
    )
    """
    If `isSuccess` = true, this contains the identifier of the invoice. If `isSuccess` = false, this contains the reason for the failure.
    """

    response_text: typing_extensions.Annotated[ResponseText, FieldMetadata(alias="responseText")]
    pageidentifier: typing.Optional[PageIdentifier] = None
    room_id: typing_extensions.Annotated[RoomIdNotInUse, FieldMetadata(alias="roomId")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .raw_client import AsyncRawStatisticClient, RawStatisticClient
from .types.stat_basic_query_record import StatBasicQueryRecord
from .types.statistics_vendor_query_record import StatisticsVendorQueryRecord
from .types.subscription_stats_query_record import SubscriptionStatsQueryRecord


class StatisticClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawStatisticClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawStatisticClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawStatisticClient
        """
        return self._raw_client

    def basic_stats(
        self,
        entry_id: int,
        freq: str,
        level: int,
        mode: str,
        *,
        end_date: typing.Optional[str] = None,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        start_date: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[StatBasicQueryRecord]:
        """
        Retrieves the basic statistics for an organization or a paypoint, for a given time period, grouped by a particular frequency.

        Parameters
        ----------
        entry_id : int
            Identifier in Payabli for the entity.

        freq : str
            Frequency to group series. Allowed values:

            - `m` - monthly
            - `w` - weekly
            - `d` - daily
            - `h` - hourly

            For example, `w` groups the results by week.

        level : int
            The entry level for the request:
              - 0 for Organization
              - 2 for Paypoint

        mode : str
            Mode for the request. Allowed values:

            - `custom` - Allows you to set a custom date range
            - `ytd` - Year To Date
            - `mtd` - Month To Date
            - `wtd` - Week To Date
            - `today` - All current day
            - `m12` - Last 12 months
            - `d30` - Last 30 days
            - `h24` - Last 24 hours
            - `lasty` - Last Year
            - `lastm` - Last Month
            - `lastw` - Last Week
            - `yesterday` - Last Day


        end_date : typing.Optional[str]
            Used with `custom` mode. The end date for the range.
            Valid formats:
              - YYYY-mm-dd
              - YYYY/mm/dd
              - mm-dd-YYYY
              - mm/dd/YYYY

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            List of parameters.

        start_date : typing.Optional[str]
            Used with `custom` mode. The start date for the range.
            Valid formats:
               - YYYY-mm-dd
               - YYYY/mm/dd
               -  mm-dd-YYYY
               - mm/dd/YYYY

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[StatBasicQueryRecord]
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.statistic.basic_stats(
            entry_id=1000000,
            freq="m",
            level=1,
            mode="ytd",
            end_date="2023-05-23",
            start_date="2023-03-23",
        )
        """
        _response = self._raw_client.basic_stats(
            entry_id,
            freq,
            level,
            mode,
            end_date=end_date,
            parameters=parameters,
            start_date=start_date,
            request_options=request_options,
        )
        return _response.data

    def customer_basic_stats(
        self,
        customer_id: int,
        freq: str,
        mode: str,
        *,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[SubscriptionStatsQueryRecord]:
        """
        Retrieves the basic statistics for a customer for a specific time period, grouped by a selected frequency.

        Parameters
        ----------
        customer_id : int
            Payabli-generated customer ID. Maps to "Customer ID" column in PartnerHub.

        freq : str
            Frequency to group series. Allowed values:

            - `m` - monthly
            - `w` - weekly
            - `d` - daily
            - `h` - hourly

            For example, `w` groups the results by week.

        mode : str
            Mode for request. Allowed values:

            - `ytd` - Year To Date
            - `mtd` - Month To Date
            - `wtd` - Week To Date
            - `today` - All current day
            - `m12` - Last 12 months
            - `d30` - Last 30 days
            - `h24` - Last 24 hours
            - `lasty` - Last Year
            - `lastm` - Last Month
            - `lastw` - Last Week
            - `yesterday` - Last Day

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            List of parameters.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[SubscriptionStatsQueryRecord]
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.statistic.customer_basic_stats(
            customer_id=998,
            freq="m",
            mode="ytd",
        )
        """
        _response = self._raw_client.customer_basic_stats(
            customer_id, freq, mode, parameters=parameters, request_options=request_options
        )
        return _response.data

    def sub_stats(
        self,
        entry_id: int,
        interval: str,
        level: int,
        *,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[StatBasicQueryRecord]:
        """
        Retrieves the subscription statistics for a given interval for a paypoint or organization.

        Parameters
        ----------
        entry_id : int
            Identifier in Payabli for the entity.

        interval : str
            Interval to get the data. Allowed values:

            - `all` - all intervals
            - `30` - 1-30 days
            - `60` - 31-60 days
            - `90` - 61-90 days
            - `plus` - +90 days

        level : int
            The entry level for the request:
              - 0 for Organization
              - 2 for Paypoint

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            List of parameters

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[StatBasicQueryRecord]
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.statistic.sub_stats(
            entry_id=1000000,
            interval="30",
            level=1,
        )
        """
        _response = self._raw_client.sub_stats(
            entry_id, interval, level, parameters=parameters, request_options=request_options
        )
        return _response.data

    def vendor_basic_stats(
        self,
        freq: str,
        id_vendor: int,
        mode: str,
        *,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[StatisticsVendorQueryRecord]:
        """
        Retrieve the basic statistics about a vendor for a given time period, grouped by frequency.

        Parameters
        ----------
        freq : str
            Frequency to group series. Allowed values:

            - `m` - monthly
            - `w` - weekly
            - `d` - daily
            - `h` - hourly

            For example, `w` groups the results by week.

        id_vendor : int
            Vendor ID.

        mode : str
            Mode for request. Allowed values:

            - `ytd` - Year To Date
            - `mtd` - Month To Date
            - `wtd` - Week To Date
            - `today` - All current day
            - `m12` - Last 12 months
            - `d30` - Last 30 days
            - `h24` - Last 24 hours
            - `lasty` - Last Year
            - `lastm` - Last Month
            - `lastw` - Last Week
            - `yesterday` - Last Day

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            List of parameters

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[StatisticsVendorQueryRecord]
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.statistic.vendor_basic_stats(
            freq="m",
            id_vendor=1,
            mode="ytd",
        )
        """
        _response = self._raw_client.vendor_basic_stats(
            freq, id_vendor, mode, parameters=parameters, request_options=request_options
        )
        return _response.data


class AsyncStatisticClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawStatisticClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawStatisticClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawStatisticClient
        """
        return self._raw_client

    async def basic_stats(
        self,
        entry_id: int,
        freq: str,
        level: int,
        mode: str,
        *,
        end_date: typing.Optional[str] = None,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        start_date: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[StatBasicQueryRecord]:
        """
        Retrieves the basic statistics for an organization or a paypoint, for a given time period, grouped by a particular frequency.

        Parameters
        ----------
        entry_id : int
            Identifier in Payabli for the entity.

        freq : str
            Frequency to group series. Allowed values:

            - `m` - monthly
            - `w` - weekly
            - `d` - daily
            - `h` - hourly

            For example, `w` groups the results by week.

        level : int
            The entry level for the request:
              - 0 for Organization
              - 2 for Paypoint

        mode : str
            Mode for the request. Allowed values:

            - `custom` - Allows you to set a custom date range
            - `ytd` - Year To Date
            - `mtd` - Month To Date
            - `wtd` - Week To Date
            - `today` - All current day
            - `m12` - Last 12 months
            - `d30` - Last 30 days
            - `h24` - Last 24 hours
            - `lasty` - Last Year
            - `lastm` - Last Month
            - `lastw` - Last Week
            - `yesterday` - Last Day


        end_date : typing.Optional[str]
            Used with `custom` mode. The end date for the range.
            Valid formats:
              - YYYY-mm-dd
              - YYYY/mm/dd
              - mm-dd-YYYY
              - mm/dd/YYYY

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            List of parameters.

        start_date : typing.Optional[str]
            Used with `custom` mode. The start date for the range.
            Valid formats:
               - YYYY-mm-dd
               - YYYY/mm/dd
               -  mm-dd-YYYY
               - mm/dd/YYYY

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[StatBasicQueryRecord]
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.statistic.basic_stats(
                entry_id=1000000,
                freq="m",
                level=1,
                mode="ytd",
                end_date="2023-05-23",
                start_date="2023-03-23",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.basic_stats(
            entry_id,
            freq,
            level,
            mode,
            end_date=end_date,
            parameters=parameters,
            start_date=start_date,
            request_options=request_options,
        )
        return _response.data

    async def customer_basic_stats(
        self,
        customer_id: int,
        freq: str,
        mode: str,
        *,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[SubscriptionStatsQueryRecord]:
        """
        Retrieves the basic statistics for a customer for a specific time period, grouped by a selected frequency.

        Parameters
        ----------
        customer_id : int
            Payabli-generated customer ID. Maps to "Customer ID" column in PartnerHub.

        freq : str
            Frequency to group series. Allowed values:

            - `m` - monthly
            - `w` - weekly
            - `d` - daily
            - `h` - hourly

            For example, `w` groups the results by week.

        mode : str
            Mode for request. Allowed values:

            - `ytd` - Year To Date
            - `mtd` - Month To Date
            - `wtd` - Week To Date
            - `today` - All current day
            - `m12` - Last 12 months
            - `d30` - Last 30 days
            - `h24` - Last 24 hours
            - `lasty` - Last Year
            - `lastm` - Last Month
            - `lastw` - Last Week
            - `yesterday` - Last Day

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            List of parameters.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[SubscriptionStatsQueryRecord]
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.statistic.customer_basic_stats(
                customer_id=998,
                freq="m",
                mode="ytd",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.customer_basic_stats(
            customer_id, freq, mode, parameters=parameters, request_options=request_options
        )
        return _response.data

    async def sub_stats(
        self,
        entry_id: int,
        interval: str,
        level: int,
        *,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[StatBasicQueryRecord]:
        """
        Retrieves the subscription statistics for a given interval for a paypoint or organization.

        Parameters
        ----------
        entry_id : int
            Identifier in Payabli for the entity.

        interval : str
            Interval to get the data. Allowed values:

            - `all` - all intervals
            - `30` - 1-30 days
            - `60` - 31-60 days
            - `90` - 61-90 days
            - `plus` - +90 days

        level : int
            The entry level for the request:
              - 0 for Organization
              - 2 for Paypoint

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            List of parameters

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[StatBasicQueryRecord]
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.statistic.sub_stats(
                entry_id=1000000,
                interval="30",
                level=1,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.sub_stats(
            entry_id, interval, level, parameters=parameters, request_options=request_options
        )
        return _response.data

    async def vendor_basic_stats(
        self,
        freq: str,
        id_vendor: int,
        mode: str,
        *,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[StatisticsVendorQueryRecord]:
        """
        Retrieve the basic statistics about a vendor for a given time period, grouped by frequency.

        Parameters
        ----------
        freq : str
            Frequency to group series. Allowed values:

            - `m` - monthly
            - `w` - weekly
            - `d` - daily
            - `h` - hourly

            For example, `w` groups the results by week.

        id_vendor : int
            Vendor ID.

        mode : str
            Mode for request. Allowed values:

            - `ytd` - Year To Date
            - `mtd` - Month To Date
            - `wtd` - Week To Date
            - `today` - All current day
            - `m12` - Last 12 months
            - `d30` - Last 30 days
            - `h24` - Last 24 hours
            - `lasty` - Last Year
            - `lastm` - Last Month
            - `lastw` - Last Week
            - `yesterday` - Last Day

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            List of parameters

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[StatisticsVendorQueryRecord]
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.statistic.vendor_basic_stats(
                freq="m",
                id_vendor=1,
                mode="ytd",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.vendor_basic_stats(
            freq, id_vendor, mode, parameters=parameters, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .apple_pay_id import ApplePayId
from .apple_pay_type import ApplePayType
from .created_at import CreatedAt
from .domain_name import DomainName
from .entry import Entry
from .last_modified import LastModified
from .paypoint_name import PaypointName


class AppleWalletData(UniversalBaseModel):
    """
    The wallet data.
    """

    entry: typing.Optional[Entry] = None
    apple_pay_merchant_id: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="applePayMerchantId")
    ] = pydantic.Field(default=None)
    """
    The Apple Pay merchant identifier.
    """

    domain_names: typing_extensions.Annotated[
        typing.Optional[typing.List[DomainName]], FieldMetadata(alias="domainNames")
    ] = pydantic.Field(default=None)
    """
    A list of domain names that are enabled for this paypoint.
    """

    paypoint_name: typing_extensions.Annotated[typing.Optional[PaypointName], FieldMetadata(alias="paypointName")] = (
        None
    )
    paypoint_url: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="paypointUrl")] = (
        pydantic.Field(default=None)
    )
    """
    The paypoint URL.
    """

    marked_for_deletion_at: typing_extensions.Annotated[
        typing.Optional[dt.datetime], FieldMetadata(alias="markedForDeletionAt")
    ] = pydantic.Field(default=None)
    """
    The date and time a paypoint's Apple Pay registration was scheduled for deletion. The paypoint will be unregistered from Apple Pay permanently 30 days from this value.
    """

    created_at: typing_extensions.Annotated[typing.Optional[CreatedAt], FieldMetadata(alias="createdAt")] = None
    updated_at: typing_extensions.Annotated[typing.Optional[LastModified], FieldMetadata(alias="updatedAt")] = None
    id: typing.Optional[ApplePayId] = pydantic.Field(default=None)
    """
    Internal ID for the Apple Pay paypoint registration update.
    """

    type: typing.Optional[ApplePayType] = pydantic.Field(default=None)
    """
    The record type, in this context it will always be `ApplePayRegistration`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class VCardGetResponseAssociatedVendorBillingData(UniversalBaseModel):
    """
    Billing data for the vendor.
    """

    id: typing.Optional[int] = pydantic.Field(default=None)
    """
    Unique identifier for billing data.
    """

    account_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="accountId")] = pydantic.Field(
        default=None
    )
    """
    Account identifier.
    """

    nickname: typing.Optional[str] = pydantic.Field(default=None)
    """
    Nickname for the account.
    """

    bank_name: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="bankName")] = pydantic.Field(
        default=None
    )
    """
    Name of the bank used for transactions.
    """

    routing_account: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="routingAccount")] = (
        pydantic.Field(default=None)
    )
    """
    Routing number for the bank account.
    """

    account_number: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="accountNumber")] = (
        pydantic.Field(default=None)
    )
    """
    Masked account number for transactions.
    """

    type_account: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="typeAccount")] = (
        pydantic.Field(default=None)
    )
    """
    Type of the bank account.
    """

    bank_account_holder_name: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="bankAccountHolderName")
    ] = pydantic.Field(default=None)
    """
    Name of the bank account holder.
    """

    bank_account_holder_type: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="bankAccountHolderType")
    ] = pydantic.Field(default=None)
    """
    Type of bank account holder.
    """

    bank_account_function: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="bankAccountFunction")
    ] = pydantic.Field(default=None)
    """
    Function of the bank account.
    """

    verified: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates if the account is verified.
    """

    status: typing.Optional[int] = pydantic.Field(default=None)
    """
    Status of the billing data.
    """

    services: typing.Optional[typing.List[typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    Services associated with the account.
    """

    default: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates if this is the default billing account.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .response_text import ResponseText


class BoardingLinkApiResponse(UniversalBaseModel):
    """ """

    response_data: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="responseData")] = (
        pydantic.Field(default=None)
    )
    """
    Reference name for boarding link (if responseText = Success) or   
    List of empty fields separated by comma (if responseText = Fail)
    """

    response_text: typing_extensions.Annotated[ResponseText, FieldMetadata(alias="responseText")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..errors.internal_server_error import InternalServerError
from ..errors.service_unavailable_error import ServiceUnavailableError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.add_payment_method_domain_api_response import AddPaymentMethodDomainApiResponse
from ..types.domain_name import DomainName
from ..types.entity_id import EntityId
from ..types.entity_type import EntityType
from ..types.payabli_api_response import PayabliApiResponse
from ..types.payment_method_domain_api_response import PaymentMethodDomainApiResponse
from ..types.payment_method_domain_general_response import PaymentMethodDomainGeneralResponse
from .types.add_payment_method_domain_request_apple_pay import AddPaymentMethodDomainRequestApplePay
from .types.add_payment_method_domain_request_google_pay import AddPaymentMethodDomainRequestGooglePay
from .types.delete_payment_method_domain_response import DeletePaymentMethodDomainResponse
from .types.list_payment_method_domains_response import ListPaymentMethodDomainsResponse
from .types.update_payment_method_domain_request_wallet import UpdatePaymentMethodDomainRequestWallet

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawPaymentMethodDomainClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def add_payment_method_domain(
        self,
        *,
        apple_pay: typing.Optional[AddPaymentMethodDomainRequestApplePay] = OMIT,
        google_pay: typing.Optional[AddPaymentMethodDomainRequestGooglePay] = OMIT,
        domain_name: typing.Optional[DomainName] = OMIT,
        entity_id: typing.Optional[EntityId] = OMIT,
        entity_type: typing.Optional[EntityType] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AddPaymentMethodDomainApiResponse]:
        """
        Add a payment method domain to an organization or paypoint.

        Parameters
        ----------
        apple_pay : typing.Optional[AddPaymentMethodDomainRequestApplePay]
            Apple Pay configuration information.

        google_pay : typing.Optional[AddPaymentMethodDomainRequestGooglePay]
            Google Pay configuration information.

        domain_name : typing.Optional[DomainName]

        entity_id : typing.Optional[EntityId]

        entity_type : typing.Optional[EntityType]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AddPaymentMethodDomainApiResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "PaymentMethodDomain",
            method="POST",
            json={
                "applePay": convert_and_respect_annotation_metadata(
                    object_=apple_pay, annotation=AddPaymentMethodDomainRequestApplePay, direction="write"
                ),
                "googlePay": convert_and_respect_annotation_metadata(
                    object_=google_pay, annotation=AddPaymentMethodDomainRequestGooglePay, direction="write"
                ),
                "domainName": domain_name,
                "entityId": entity_id,
                "entityType": entity_type,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AddPaymentMethodDomainApiResponse,
                    parse_obj_as(
                        type_=AddPaymentMethodDomainApiResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def cascade_payment_method_domain(
        self, domain_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[PaymentMethodDomainGeneralResponse]:
        """
        Cascades a payment method domain to all child entities. All paypoints and suborganization under this parent will inherit this domain and its settings.

        Parameters
        ----------
        domain_id : str
            The payment method domain's ID in Payabli.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PaymentMethodDomainGeneralResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"PaymentMethodDomain/{jsonable_encoder(domain_id)}/cascade",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PaymentMethodDomainGeneralResponse,
                    parse_obj_as(
                        type_=PaymentMethodDomainGeneralResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete_payment_method_domain(
        self, domain_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DeletePaymentMethodDomainResponse]:
        """
        Delete a payment method domain. You can't delete an inherited domain, you must delete a domain at the organization level.

        Parameters
        ----------
        domain_id : str
            The payment method domain's ID in Payabli.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DeletePaymentMethodDomainResponse]
            Success response for a deleted payment method domain.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"PaymentMethodDomain/{jsonable_encoder(domain_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeletePaymentMethodDomainResponse,
                    parse_obj_as(
                        type_=DeletePaymentMethodDomainResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_payment_method_domain(
        self, domain_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[PaymentMethodDomainApiResponse]:
        """
        Get the details for a payment method domain.

        Parameters
        ----------
        domain_id : str
            The payment method domain's ID in Payabli.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PaymentMethodDomainApiResponse]
            Success response that includes a payment method domain's details.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"PaymentMethodDomain/{jsonable_encoder(domain_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PaymentMethodDomainApiResponse,
                    parse_obj_as(
                        type_=PaymentMethodDomainApiResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def list_payment_method_domains(
        self,
        *,
        entity_id: typing.Optional[int] = None,
        entity_type: typing.Optional[str] = None,
        from_record: typing.Optional[int] = None,
        limit_record: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ListPaymentMethodDomainsResponse]:
        """
        Get a list of payment method domains that belong to a PSP, organization, or paypoint.

        Parameters
        ----------
        entity_id : typing.Optional[int]
            Identifier for the organization or paypoint.
            - For organization, provide the organization ID - For paypoint, provide the paypoint ID

        entity_type : typing.Optional[str]
            The type of entity. Valid values:
              - organization
              - paypoint
              - psp

        from_record : typing.Optional[int]
            Number of records to skip. Defaults to `0`.

        limit_record : typing.Optional[int]
            Max number of records for query response. Defaults to `20`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ListPaymentMethodDomainsResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "PaymentMethodDomain/list",
            method="GET",
            params={
                "entityId": entity_id,
                "entityType": entity_type,
                "fromRecord": from_record,
                "limitRecord": limit_record,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListPaymentMethodDomainsResponse,
                    parse_obj_as(
                        type_=ListPaymentMethodDomainsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_payment_method_domain(
        self,
        domain_id: str,
        *,
        apple_pay: typing.Optional[UpdatePaymentMethodDomainRequestWallet] = OMIT,
        google_pay: typing.Optional[UpdatePaymentMethodDomainRequestWallet] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PaymentMethodDomainGeneralResponse]:
        """
        Update a payment method domain's configuration values.

        Parameters
        ----------
        domain_id : str
            The payment method domain's ID in Payabli.

        apple_pay : typing.Optional[UpdatePaymentMethodDomainRequestWallet]

        google_pay : typing.Optional[UpdatePaymentMethodDomainRequestWallet]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PaymentMethodDomainGeneralResponse]
            Success response for configuration update.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"PaymentMethodDomain/{jsonable_encoder(domain_id)}",
            method="PATCH",
            json={
                "applePay": convert_and_respect_annotation_metadata(
                    object_=apple_pay, annotation=UpdatePaymentMethodDomainRequestWallet, direction="write"
                ),
                "googlePay": convert_and_respect_annotation_metadata(
                    object_=google_pay, annotation=UpdatePaymentMethodDomainRequestWallet, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PaymentMethodDomainGeneralResponse,
                    parse_obj_as(
                        type_=PaymentMethodDomainGeneralResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def verify_payment_method_domain(
        self, domain_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[PaymentMethodDomainGeneralResponse]:
        """
        Verify a new payment method domain. If verification is successful, Apple Pay is automatically activated for the domain.

        Parameters
        ----------
        domain_id : str
            The payment method domain's ID in Payabli.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PaymentMethodDomainGeneralResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"PaymentMethodDomain/{jsonable_encoder(domain_id)}/verify",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PaymentMethodDomainGeneralResponse,
                    parse_obj_as(
                        type_=PaymentMethodDomainGeneralResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawPaymentMethodDomainClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def add_payment_method_domain(
        self,
        *,
        apple_pay: typing.Optional[AddPaymentMethodDomainRequestApplePay] = OMIT,
        google_pay: typing.Optional[AddPaymentMethodDomainRequestGooglePay] = OMIT,
        domain_name: typing.Optional[DomainName] = OMIT,
        entity_id: typing.Optional[EntityId] = OMIT,
        entity_type: typing.Optional[EntityType] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AddPaymentMethodDomainApiResponse]:
        """
        Add a payment method domain to an organization or paypoint.

        Parameters
        ----------
        apple_pay : typing.Optional[AddPaymentMethodDomainRequestApplePay]
            Apple Pay configuration information.

        google_pay : typing.Optional[AddPaymentMethodDomainRequestGooglePay]
            Google Pay configuration information.

        domain_name : typing.Optional[DomainName]

        entity_id : typing.Optional[EntityId]

        entity_type : typing.Optional[EntityType]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AddPaymentMethodDomainApiResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "PaymentMethodDomain",
            method="POST",
            json={
                "applePay": convert_and_respect_annotation_metadata(
                    object_=apple_pay, annotation=AddPaymentMethodDomainRequestApplePay, direction="write"
                ),
                "googlePay": convert_and_respect_annotation_metadata(
                    object_=google_pay, annotation=AddPaymentMethodDomainRequestGooglePay, direction="write"
                ),
                "domainName": domain_name,
                "entityId": entity_id,
                "entityType": entity_type,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AddPaymentMethodDomainApiResponse,
                    parse_obj_as(
                        type_=AddPaymentMethodDomainApiResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def cascade_payment_method_domain(
        self, domain_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PaymentMethodDomainGeneralResponse]:
        """
        Cascades a payment method domain to all child entities. All paypoints and suborganization under this parent will inherit this domain and its settings.

        Parameters
        ----------
        domain_id : str
            The payment method domain's ID in Payabli.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PaymentMethodDomainGeneralResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"PaymentMethodDomain/{jsonable_encoder(domain_id)}/cascade",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PaymentMethodDomainGeneralResponse,
                    parse_obj_as(
                        type_=PaymentMethodDomainGeneralResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete_payment_method_domain(
        self, domain_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DeletePaymentMethodDomainResponse]:
        """
        Delete a payment method domain. You can't delete an inherited domain, you must delete a domain at the organization level.

        Parameters
        ----------
        domain_id : str
            The payment method domain's ID in Payabli.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DeletePaymentMethodDomainResponse]
            Success response for a deleted payment method domain.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"PaymentMethodDomain/{jsonable_encoder(domain_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeletePaymentMethodDomainResponse,
                    parse_obj_as(
                        type_=DeletePaymentMethodDomainResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_payment_method_domain(
        self, domain_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PaymentMethodDomainApiResponse]:
        """
        Get the details for a payment method domain.

        Parameters
        ----------
        domain_id : str
            The payment method domain's ID in Payabli.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PaymentMethodDomainApiResponse]
            Success response that includes a payment method domain's details.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"PaymentMethodDomain/{jsonable_encoder(domain_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PaymentMethodDomainApiResponse,
                    parse_obj_as(
                        type_=PaymentMethodDomainApiResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def list_payment_method_domains(
        self,
        *,
        entity_id: typing.Optional[int] = None,
        entity_type: typing.Optional[str] = None,
        from_record: typing.Optional[int] = None,
        limit_record: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ListPaymentMethodDomainsResponse]:
        """
        Get a list of payment method domains that belong to a PSP, organization, or paypoint.

        Parameters
        ----------
        entity_id : typing.Optional[int]
            Identifier for the organization or paypoint.
            - For organization, provide the organization ID - For paypoint, provide the paypoint ID

        entity_type : typing.Optional[str]
            The type of entity. Valid values:
              - organization
              - paypoint
              - psp

        from_record : typing.Optional[int]
            Number of records to skip. Defaults to `0`.

        limit_record : typing.Optional[int]
            Max number of records for query response. Defaults to `20`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ListPaymentMethodDomainsResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "PaymentMethodDomain/list",
            method="GET",
            params={
                "entityId": entity_id,
                "entityType": entity_type,
                "fromRecord": from_record,
                "limitRecord": limit_record,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListPaymentMethodDomainsResponse,
                    parse_obj_as(
                        type_=ListPaymentMethodDomainsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_payment_method_domain(
        self,
        domain_id: str,
        *,
        apple_pay: typing.Optional[UpdatePaymentMethodDomainRequestWallet] = OMIT,
        google_pay: typing.Optional[UpdatePaymentMethodDomainRequestWallet] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PaymentMethodDomainGeneralResponse]:
        """
        Update a payment method domain's configuration values.

        Parameters
        ----------
        domain_id : str
            The payment method domain's ID in Payabli.

        apple_pay : typing.Optional[UpdatePaymentMethodDomainRequestWallet]

        google_pay : typing.Optional[UpdatePaymentMethodDomainRequestWallet]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PaymentMethodDomainGeneralResponse]
            Success response for configuration update.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"PaymentMethodDomain/{jsonable_encoder(domain_id)}",
            method="PATCH",
            json={
                "applePay": convert_and_respect_annotation_metadata(
                    object_=apple_pay, annotation=UpdatePaymentMethodDomainRequestWallet, direction="write"
                ),
                "googlePay": convert_and_respect_annotation_metadata(
                    object_=google_pay, annotation=UpdatePaymentMethodDomainRequestWallet, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PaymentMethodDomainGeneralResponse,
                    parse_obj_as(
                        type_=PaymentMethodDomainGeneralResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def verify_payment_method_domain(
        self, domain_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PaymentMethodDomainGeneralResponse]:
        """
        Verify a new payment method domain. If verification is successful, Apple Pay is automatically activated for the domain.

        Parameters
        ----------
        domain_id : str
            The payment method domain's ID in Payabli.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PaymentMethodDomainGeneralResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"PaymentMethodDomain/{jsonable_encoder(domain_id)}/verify",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PaymentMethodDomainGeneralResponse,
                    parse_obj_as(
                        type_=PaymentMethodDomainGeneralResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.is_success import IsSuccess
from ...types.page_identifier import PageIdentifier
from ...types.response_text import ResponseText
from .response_data_refunds import ResponseDataRefunds


class RefundWithInstructionsResponse(UniversalBaseModel):
    response_text: typing_extensions.Annotated[ResponseText, FieldMetadata(alias="responseText")]
    is_success: typing_extensions.Annotated[IsSuccess, FieldMetadata(alias="isSuccess")]
    response_data: typing_extensions.Annotated[ResponseDataRefunds, FieldMetadata(alias="responseData")]
    pageidentifier: typing.Optional[PageIdentifier] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.file_content_ftype import FileContentFtype


class FileContentImageOnly(UniversalBaseModel):
    ftype: typing.Optional[FileContentFtype] = None
    filename: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the file to be uploaded
    """

    furl: typing.Optional[str] = pydantic.Field(default=None)
    """
    Optional URL link to the file
    """

    f_content: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="fContent")] = pydantic.Field(
        default=None
    )
    """
    Base64-encoded file content
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

Frequency = typing.Union[
    typing.Literal["one-time", "weekly", "every2weeks", "every6months", "monthly", "every3months", "annually"],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.entrypointfield import Entrypointfield
from .notification_request import NotificationRequest


class PaypointMoveRequest(UniversalBaseModel):
    entry_point: typing_extensions.Annotated[Entrypointfield, FieldMetadata(alias="entryPoint")]
    new_parent_organization_id: typing_extensions.Annotated[int, FieldMetadata(alias="newParentOrganizationId")] = (
        pydantic.Field()
    )
    """
    The ID for the paypoint's new parent organization.
    """

    notification_request: typing_extensions.Annotated[
        typing.Optional[NotificationRequest], FieldMetadata(alias="notificationRequest")
    ] = pydantic.Field(default=None)
    """
    Optional notification request object for a webhook
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .template_element import TemplateElement
from .visible import Visible


class AchFeeSection(UniversalBaseModel):
    advanced_settlement_ach_fee: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="advancedSettlementAchFee")
    ] = None
    annual_ach_fee: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="annualAchFee")
    ] = None
    chargeback_ach_fee: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="chargebackAchFee")
    ] = None
    early_termination_ach_fee: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="earlyTerminationAchFee")
    ] = None
    monthly_ach_fee: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="monthlyAchFee")
    ] = None
    quarterly_pci_ach_fee: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="quarterlyPCIAchFee")
    ] = None
    returned_ach_fee: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="returnedAchFee")
    ] = None
    same_day_ach_fee: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="sameDayAchFee")
    ] = None
    sunday_origination_ach_fee: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="sundayOriginationAchFee")
    ] = None
    verify_bank_ach_fee: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="verifyBankAchFee")
    ] = None
    verify_fund_ach_fee: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="verifyFundAchFee")
    ] = None
    verify_negative_ach_fee: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="verifyNegativeAchFee")
    ] = None
    visible: typing.Optional[Visible] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .template_additional_data_section import TemplateAdditionalDataSection
from .template_element import TemplateElement
from .visible import Visible


class SignerSection(UniversalBaseModel):
    visible: typing.Optional[Visible] = None
    name: typing.Optional[TemplateElement] = None
    ssn: typing.Optional[TemplateElement] = None
    dob: typing.Optional[TemplateElement] = None
    phone: typing.Optional[TemplateElement] = None
    email: typing.Optional[TemplateElement] = None
    address: typing.Optional[TemplateElement] = None
    address_1: typing_extensions.Annotated[typing.Optional[TemplateElement], FieldMetadata(alias="address1")] = None
    city: typing.Optional[TemplateElement] = None
    country: typing.Optional[TemplateElement] = None
    state: typing.Optional[TemplateElement] = None
    zip: typing.Optional[TemplateElement] = None
    acceptance: typing.Optional[TemplateElement] = None
    signed_document_reference: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="signedDocumentReference")
    ] = None
    additional_data: typing_extensions.Annotated[
        typing.Optional[TemplateAdditionalDataSection], FieldMetadata(alias="additionalData")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .sales_code import SalesCode


class SalesSection(UniversalBaseModel):
    sales_code: typing_extensions.Annotated[typing.Optional[SalesCode], FieldMetadata(alias="salesCode")] = None
    sales_crm: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="salesCRM")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class BillingFeeDetail(UniversalBaseModel):
    billable_event: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="billableEvent")] = None
    service: typing.Optional[str] = None
    event_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="eventId")] = None
    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Description of the billing fee
    """

    category: typing.Optional[str] = pydantic.Field(default=None)
    """
    Category of the billing fee
    """

    fix_price: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="fixPrice")] = pydantic.Field(
        default=None
    )
    """
    Fixed price component of the fee
    """

    float_price: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="floatPrice")] = (
        pydantic.Field(default=None)
    )
    """
    Percentage component of the fee
    """

    billable_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="billableAmount")] = (
        pydantic.Field(default=None)
    )
    """
    Amount eligible for the fee
    """

    bill_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="billAmount")] = (
        pydantic.Field(default=None)
    )
    """
    Total fee amount charged
    """

    frequency: typing.Optional[str] = None
    service_group: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="serviceGroup")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .ach_holder_type import AchHolderType
from .ach_sec_code import AchSecCode
from .batch_number import BatchNumber
from .bill_data import BillData
from .datetime_nullable import DatetimeNullable
from .dbaname import Dbaname
from .device import Device
from .entrypage_id import EntrypageId
from .entrypointfield import Entrypointfield
from .external_processor_information import ExternalProcessorInformation
from .fee_amount import FeeAmount
from .legalname import Legalname
from .netamountnullable import Netamountnullable
from .operation import Operation
from .order_id import OrderId
from .org_parent_name import OrgParentName
from .orgid import Orgid
from .payor_id import PayorId
from .pending_fee_amount import PendingFeeAmount
from .query_c_fee_transaction import QueryCFeeTransaction
from .query_payment_data import QueryPaymentData
from .query_response_data import QueryResponseData
from .query_transaction_events import QueryTransactionEvents
from .query_transaction_payor_data import QueryTransactionPayorData
from .refund_id import RefundId
from .returned_id import ReturnedId
from .source import Source
from .split_funding import SplitFunding


class TransactionQueryRecords(UniversalBaseModel):
    ach_holder_type: typing_extensions.Annotated[
        typing.Optional[AchHolderType], FieldMetadata(alias="AchHolderType")
    ] = None
    ach_sec_code: typing_extensions.Annotated[typing.Optional[AchSecCode], FieldMetadata(alias="AchSecCode")] = None
    batch_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="BatchAmount")] = (
        pydantic.Field(default=None)
    )
    """
    Batch amount.
    """

    batch_number: typing_extensions.Annotated[typing.Optional[BatchNumber], FieldMetadata(alias="BatchNumber")] = None
    cfee_transactions: typing_extensions.Annotated[
        typing.Optional[typing.List[QueryCFeeTransaction]], FieldMetadata(alias="CfeeTransactions")
    ] = pydantic.Field(default=None)
    """
    Service Fee or sub-charge transaction associated to the main transaction.
    """

    connector_name: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="ConnectorName")] = (
        pydantic.Field(default=None)
    )
    """
    Connector used for transaction.
    """

    customer: typing_extensions.Annotated[
        typing.Optional[QueryTransactionPayorData], FieldMetadata(alias="Customer")
    ] = None
    device_id: typing_extensions.Annotated[typing.Optional[Device], FieldMetadata(alias="DeviceId")] = None
    entrypage_id: typing_extensions.Annotated[typing.Optional[EntrypageId], FieldMetadata(alias="EntrypageId")] = None
    external_processor_information: typing_extensions.Annotated[
        typing.Optional[ExternalProcessorInformation], FieldMetadata(alias="ExternalProcessorInformation")
    ] = None
    fee_amount: typing_extensions.Annotated[typing.Optional[FeeAmount], FieldMetadata(alias="FeeAmount")] = None
    gateway_trans_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="GatewayTransId")] = (
        pydantic.Field(default=None)
    )
    """
    Internal identifier used for processing.
    """

    invoice_data: typing_extensions.Annotated[typing.Optional[BillData], FieldMetadata(alias="InvoiceData")] = None
    method: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Method")] = pydantic.Field(
        default=None
    )
    """
    Payment method used: card, ach, or wallet.
    """

    net_amount: typing_extensions.Annotated[typing.Optional[Netamountnullable], FieldMetadata(alias="NetAmount")] = (
        pydantic.Field(default=None)
    )
    """
    Net amount paid.
    """

    operation: typing_extensions.Annotated[typing.Optional[Operation], FieldMetadata(alias="Operation")] = None
    order_id: typing_extensions.Annotated[typing.Optional[OrderId], FieldMetadata(alias="OrderId")] = None
    org_id: typing_extensions.Annotated[typing.Optional[Orgid], FieldMetadata(alias="OrgId")] = pydantic.Field(
        default=None
    )
    """
    ID of immediate parent organization.
    """

    parent_org_name: typing_extensions.Annotated[
        typing.Optional[OrgParentName], FieldMetadata(alias="ParentOrgName")
    ] = None
    payment_data: typing_extensions.Annotated[typing.Optional[QueryPaymentData], FieldMetadata(alias="PaymentData")] = (
        None
    )
    payment_trans_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="PaymentTransId")] = (
        pydantic.Field(default=None)
    )
    """
    Unique Transaction ID.
    """

    payor_id: typing_extensions.Annotated[typing.Optional[PayorId], FieldMetadata(alias="PayorId")] = None
    paypoint_dbaname: typing_extensions.Annotated[typing.Optional[Dbaname], FieldMetadata(alias="PaypointDbaname")] = (
        pydantic.Field(default=None)
    )
    """
    Paypoint's DBA name.
    """

    paypoint_entryname: typing_extensions.Annotated[
        typing.Optional[Entrypointfield], FieldMetadata(alias="PaypointEntryname")
    ] = pydantic.Field(default=None)
    """
    Paypoint's entryname.
    """

    paypoint_id: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="PaypointId")] = pydantic.Field(
        default=None
    )
    """
    InternalId for paypoint.
    """

    paypoint_legalname: typing_extensions.Annotated[
        typing.Optional[Legalname], FieldMetadata(alias="PaypointLegalname")
    ] = pydantic.Field(default=None)
    """
    Paypoint's legal name.
    """

    pending_fee_amount: typing_extensions.Annotated[
        typing.Optional[PendingFeeAmount], FieldMetadata(alias="PendingFeeAmount")
    ] = None
    refund_id: typing_extensions.Annotated[typing.Optional[RefundId], FieldMetadata(alias="RefundId")] = None
    response_data: typing_extensions.Annotated[
        typing.Optional[QueryResponseData], FieldMetadata(alias="ResponseData")
    ] = None
    returned_id: typing_extensions.Annotated[typing.Optional[ReturnedId], FieldMetadata(alias="ReturnedId")] = None
    schedule_reference: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="ScheduleReference")] = (
        pydantic.Field(default=None)
    )
    """
    Reference to the subscription that originated the transaction.
    """

    settlement_status: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="SettlementStatus")] = (
        pydantic.Field(default=None)
    )
    """
    Settlement status for transaction. See [the docs](/developers/references/money-in-statuses#payment-funding-status) for a full reference.
    """

    source: typing_extensions.Annotated[typing.Optional[Source], FieldMetadata(alias="Source")] = None
    split_funding_instructions: typing_extensions.Annotated[
        typing.Optional[SplitFunding], FieldMetadata(alias="splitFundingInstructions")
    ] = None
    total_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="TotalAmount")] = (
        pydantic.Field(default=None)
    )
    """
    Transaction total amount (including service fee or sub-charge)
    """

    transaction_events: typing_extensions.Annotated[
        typing.Optional[typing.List[QueryTransactionEvents]], FieldMetadata(alias="TransactionEvents")
    ] = pydantic.Field(default=None)
    """
    Events associated with this transaction.
    """

    transaction_time: typing_extensions.Annotated[
        typing.Optional[DatetimeNullable], FieldMetadata(alias="TransactionTime")
    ] = pydantic.Field(default=None)
    """
    Transaction date and time, in UTC.
    """

    trans_additional_data: typing_extensions.Annotated[
        typing.Optional[typing.Optional[typing.Any]], FieldMetadata(alias="TransAdditionalData")
    ] = None
    trans_status: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="TransStatus")] = (
        pydantic.Field(default=None)
    )
    """
    Status of transaction. See [the docs](/developers/references/money-in-statuses#money-in-transaction-status) for a full reference.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .ach_setup import AchSetup
from .card_setup import CardSetup
from .odp_setup import OdpSetup


class Services(UniversalBaseModel):
    """
    Controls which services will be enabled for the merchant.
    """

    ach: typing.Optional[AchSetup] = None
    card: typing.Optional[CardSetup] = None
    odp: typing.Optional[OdpSetup] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class GooglePayMetadata(UniversalBaseModel):
    """
    This metadata appears only when the domain verification check fails. It gives more information about why the check failed.
    """

    status_code: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="statusCode")] = pydantic.Field(
        default=None
    )
    """
    The status code return by the domain verification URL.
    """

    redirect_url: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="redirectUrl")] = (
        pydantic.Field(default=None)
    )
    """
    If the domain verification URL is redirected, this is the URL it's redirected to.  For example, www.partner.com could redirect to www.partners-new-home-page.com. In this case, you should add www.partners-new-home-page.com as a domain instead of www.partner.com.
    """

    redirect_domain_name: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="redirectDomainName")
    ] = pydantic.Field(default=None)
    """
    The domain name if the domain verification URL returns a redirect.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .additional_data_string import AdditionalDataString
from .created_at import CreatedAt
from .email import Email
from .language import Language
from .mfa import Mfa
from .mfa_mode import MfaMode
from .name_user import NameUser
from .org_x_scope import OrgXScope
from .phone_number import PhoneNumber
from .timezone import Timezone
from .usr_access import UsrAccess
from .usr_status import UsrStatus


class UserQueryRecord(UniversalBaseModel):
    access: typing_extensions.Annotated[typing.Optional[typing.List[UsrAccess]], FieldMetadata(alias="Access")] = None
    additional_data: typing_extensions.Annotated[
        typing.Optional[AdditionalDataString], FieldMetadata(alias="AdditionalData")
    ] = None
    created_at: typing_extensions.Annotated[typing.Optional[CreatedAt], FieldMetadata(alias="createdAt")] = (
        pydantic.Field(default=None)
    )
    """
    The timestamp for the user's creation, in UTC.
    """

    email: typing_extensions.Annotated[typing.Optional[Email], FieldMetadata(alias="Email")] = pydantic.Field(
        default=None
    )
    """
    The user's email address.
    """

    language: typing.Optional[Language] = None
    last_access: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="lastAccess")] = (
        pydantic.Field(default=None)
    )
    """
    The timestamp for the user's last activity, in UTC.
    """

    name: typing_extensions.Annotated[typing.Optional[NameUser], FieldMetadata(alias="Name")] = None
    phone: typing_extensions.Annotated[typing.Optional[PhoneNumber], FieldMetadata(alias="Phone")] = pydantic.Field(
        default=None
    )
    """
    The user's phone number.
    """

    scope: typing_extensions.Annotated[typing.Optional[typing.List[OrgXScope]], FieldMetadata(alias="Scope")] = None
    sn_data: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="snData")] = pydantic.Field(
        default=None
    )
    """
    Additional data provided by the social network related to the customer.
    """

    sn_identifier: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="snIdentifier")] = (
        pydantic.Field(default=None)
    )
    """
    Identifier or token for customer in linked social network.
    """

    sn_provider: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="snProvider")] = pydantic.Field(
        default=None
    )
    """
    Social network linked to customer. Possible values: facebook, google, twitter, microsoft.
    """

    time_zone: typing_extensions.Annotated[typing.Optional[Timezone], FieldMetadata(alias="timeZone")] = None
    user_id: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="userId")] = pydantic.Field(
        default=None
    )
    """
    The user's ID in Payabli.
    """

    usr_mfa: typing_extensions.Annotated[typing.Optional[Mfa], FieldMetadata(alias="UsrMFA")] = None
    usr_mfa_mode: typing_extensions.Annotated[typing.Optional[MfaMode], FieldMetadata(alias="UsrMFAMode")] = None
    usr_status: typing_extensions.Annotated[typing.Optional[UsrStatus], FieldMetadata(alias="UsrStatus")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .order import Order


class PayCategory(UniversalBaseModel):
    description: typing.Optional[str] = None
    label: typing.Optional[str] = None
    name: typing.Optional[str] = None
    optional_pay: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="optionalPay")] = None
    order: typing.Optional[Order] = None
    quantity: typing.Optional[int] = None
    show_description: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="showDescription")] = None
    type: typing.Optional[str] = None
    value: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ...types.notification_report_request import NotificationReportRequest
from ...types.notification_standard_request import NotificationStandardRequest

AddNotificationRequest = typing.Union[NotificationStandardRequest, NotificationReportRequest]
